/*
 * Copyright (C) 2000 Bang Jun-Young
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THE AUTHOR ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION,
 * AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 */

#include <string.h>

int
utf16_to_utf8(int src, unsigned char *dest)
{
	unsigned char byte[4];
	int i, ch, nbytes;

	if (src < 0x80) {
		nbytes = 1;
		byte[0] = src;
	} else if (src < 0x800) {
		nbytes = 2;
		byte[1] = (src & 0x3f) | 0x80;
		byte[0] = ((src << 2) & 0xcf00 | 0xc000) >> 8;
	} else {
		nbytes = 3;
		byte[2] = (src & 0x3f) | 0x80;
		byte[1] = ((src << 2) & 0x3f00 | 0x8000) >> 8;
		byte[0] = ((src << 4) & 0x3f0000 | 0xe00000) >> 16;
	}

	for (i = nbytes; i < 4; i++)
		byte[i] = 0;

	strcpy(dest, byte);

	return nbytes;
}

int
utf8_to_utf16(unsigned char *src, int *dest)
{
	int nbytes;

	if (*src < 0x80) {
		/* U+0000 - U+007F: plain ASCII */
		nbytes = 1;
		*dest = *src;
	} else if (*src > 0xe0 && *(src + 1) > 0x80 && *(src + 2) > 0x80) {
		/* U+0800 - U+FFFF */
		nbytes = 3;
		*dest = ((*src & 0x0f) << 12) | ((*(src + 1) & 0x7c) << 6) | 
		       ((*(src + 1) & 0x03) << 6) | (*(src + 2) & 0x3f);
	} else if (*src > 0xc0 && *(src + 1) > 0x80) {
		/* U+0080 - U+07FF */
		nbytes = 2;
		*dest = ((*src & 0x0f) << 6) | (*(src + 1) & 0x3f);
	} else
		return 0;

	return nbytes;
}

#if 0 /* Set to 1 if you want to test above functions. */
void convtest(int, unsigned char *);
void convtest2(unsigned char *, int);

main()
{
	int a = 0xa9;
	int b = 0x2260;
	char d[4], e[4];

	printf("utf16_to_utf8\n");
	convtest(a, d);
	convtest(b, e);

	printf("utf8_to_utf16\n");
	convtest2("\xe2\x89\xa0", a);
	convtest2("\xc2\xa9", b);
}

void
convtest(int a, unsigned char *b)
{
	int n;

	n = utf16_to_utf8(a, b);
	printf("%s (len = %d)\n", b, n);
}

void
convtest2(unsigned char *a, int b)
{
	int n;

	n = utf8_to_utf16(a, &b);
	printf("0x%04x (len = %d)\n", b, n);	
}
#endif

