/*
 * Copyright (C) 2000 Bang Jun-Young
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THE AUTHOR ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION,
 * AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 */

#define UNI_HANGUL_FIRST 	0xac00
#define UNI_HANGUL_LAST		0xd7a3

#define NUM_CHOSEONG		19
#define NUM_JOONGSEONG		21
#define NUM_JONGSEONG		28

/*
 * utf16_to_johab: Convert UTF-16 Hangul code to Johab code.
 * Note that Johab code used here is different from the popular Johab code.
 */

int
utf16_to_johab(int ch)
{
	int choseong, joongseong, jongseong;

	ch -= UNI_HANGUL_FIRST;
	choseong = ch / (NUM_JOONGSEONG * NUM_JONGSEONG);
	joongseong = (ch % (NUM_JOONGSEONG * NUM_JONGSEONG)) / NUM_JONGSEONG;
	jongseong = ch % NUM_JONGSEONG;

	return (choseong << 10) | (joongseong << 5) | jongseong;
}

/*
 * johab_to_utf16: Convert Hangul Johab code to UTF-16 code.
 * Note that Johab code used here is different from the popular Johab code.
 */

int
johab_to_utf16(int ch)
{
	int choseong, joongseong, jongseong;

	choseong = (ch >> 10) & 0x1f;
	joongseong = (ch >> 5) & 0x1f;
	jongseong = ch & 0x1f;

	choseong *= (NUM_JOONGSEONG * NUM_JONGSEONG);
	joongseong *= NUM_JONGSEONG;

	return UNI_HANGUL_FIRST + choseong + joongseong + jongseong;
}

#if 0
main()
{
	int a, b;
	a = 0x20;		/* 00000 00001 00000 = G AE _ */
	b = johab_to_utf16(a);
	printf("U+0x%04X\n", b);
	a = 0x400;		/* 00001 00000 00000 = GG A _ */
	b = johab_to_utf16(a);
	printf("U+0x%04X\n", b);
}
#endif

